DartGap.MessageHandler = function() {
  	return {
  		"accelerometer": function(message, callback) {
  			function errorCallback(err) {
    			alert("Accelerometer error: " + err.code);
			}
			switch(message.type) {
				case "current":
					navigator.accelerometer.getCurrentAcceleration(callback, errorCallback);
			    	break;
			    default:
			    	throw "unhandled accelerometer message type " + message.type;
			}
	  	},
		"database": function(message, callback) {
		    var db = cache[message.content.connectionId],
		    	executeSql, 
		    	queries, 
		    	queryResult;

		    function getResultCollector(queryNumber) {
		    	var resultCollector = function(tx, resultSet) {
		    		var length = resultSet.rows.length;
		    		var result = {
		    			"query": queries[queryNumber],
		    			"result": new Array(length)
		    		} 
		    		if(length > 0) {
		    			for(var i=0; i<length; i++) {
		    				result["result"][i] = resultSet.rows.item(i)	
		    			}
				    } 

				    queryResult[queryNumber] = result;
				};
				return resultCollector;
			}

			function errorCallback(err) {
    			alert("Error processing SQL: " + err.code);
			}

			function successCallback() {
				callback(queryResult);
			}    
		    
		    switch(message.type) {
		    	case "changeVersion":
		    		db.changeVersion(message.content.expectedCurrentVersion, message.content.newVersion);
		    		callback();
    				break;
    			case "closeDatabase":
    				delete cache[message.content.connectionId];
    				callback();
    				break;
		    	case "executeBatch": 
					executeSql = function(tx) {
						queries = message.content["queries"];
						queryResult = new Array(queries.length);
						for(var i=0; i < queries.length; i++) {		
							tx.executeSql(queries[i], [], getResultCollector(i), errorCallback); 
						}
					}; 
					db.transaction(executeSql, errorCallback, successCallback);	
			        break;
			    default:
			    	throw "unhandled database message type " + message.type;
		    }
	  	}, 
	  	"device": function(message, callback) {
		    var db, result;
		    switch(message.type) {
				case "openDatabase": 
					db = window.openDatabase(message.content.name, message.content.version, message.content.displayName, message.content.size);
					cache[message.content.connectionId] = db;
			        callback();
			        break;
			    default:
			    	throw "unhandled device message type " + message.type;
		    }
		}, 
	  	"notification": function(message, callback) {
			switch(message.type) {
				case "alert":
			    	navigator.notification.alert(message.content.alert, callback);
			    	break;
			    case "beep":
			    	navigator.notification.beep(message.content.times);
			    	break;
			    case "confirm":
			    	navigator.notification.confirm(message.content.dialog, callback, message.content.title, message.content.labels);
			    	break;
			    case "bibrate":
			    	navigator.notification.vibrate(message.content.duration);
			    	break;
			    default:
			    	throw "unhandled notification message type " + message.type;
			}
	  	}
	};
};
