// Copyright (c) 2013, the project authors. Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed 
// by a Apache license that can be found in the LICENSE file.

part of handheld_sample;

class AccelerometerPage extends Page {
  AccelerometerPage(Router router): super(router, pageTitle:"Accelerometer API", showBackButton:true);
  
  _showPageContent(Element page, Device device) { 
    var accelerometer = device.accelerometer;
    page.innerHtml = ''' 
      <ul>
        <li><button id='get_acceleration'>Get acceleration</button></li>
        <li><button id='watch_acceleration'>Watch acceleration</button></li>
        <li><button id='stop_watch'>Stop watch</button></li>
        <li><button id='clear_acceleration'>Clear acceleration</button></li>
      </ul>
      <div id='acceleration_result'></div>
    ''';
    var resultContainer = page.query('#acceleration_result');
    
    _handleGetAcceleration(page, resultContainer, accelerometer);
    _handleWatchAcceleration(page, resultContainer, accelerometer);
    _handleClearAcceleration(page, resultContainer);
  }
  
  _handleGetAcceleration(Element page, Element resultContainer, DeviceAccelerometer accelerometer) {
    page.query('#get_acceleration').onClick.listen((e) {
      accelerometer.currentAcceleration.then((DeviceAcceleration acceleration) {
        _showAcceleration(resultContainer, acceleration);
      });
    });
  }
  
  _handleWatchAcceleration(Element page, Element resultContainer, DeviceAccelerometer accelerometer) {
    DeviceAccelerationStream stream;
    page.query('#watch_acceleration').onClick.listen((e) {
      stream = accelerometer.watchAcceleration(1000);
      stream.listen((DeviceAcceleration acceleration) {
        _showAcceleration(resultContainer, acceleration);
      });
    });
    
    page.query('#stop_watch').onClick.listen((e) {
      if(stream != null) {
        stream.close();
      }
    });
  }
  
  _handleClearAcceleration(Element page, Element resultContainer) {
    page.query('#clear_acceleration').onClick.listen((e) {
      resultContainer.nodes.clear();
    });
  }
  
  _showAcceleration(Element container, DeviceAcceleration acceleration) {
    container.innerHtml = ''' 
      <ul>
        <li><p><strong>x:</strong> ${acceleration.x}</p></li>
        <li><p><strong>y:</strong> ${acceleration.y}</p></li>
        <li><p><strong>z:</strong> ${acceleration.z}</p></li>
        <li><p><strong>timestamp:</strong> ${acceleration.timestamp}</p></li>
      </ul>
    ''';
  }
}









