// Copyright (c) 2013, the project authors. Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed 
// by a Apache license that can be found in the LICENSE file.

/** Utilities for building and deploying Dart apps on PhoneGap devices */
library handheld_builder;

import 'dart:async';
import 'dart:io';
import 'package:log4dart/log4dart.dart';

/** Compile Dart project from [inputFilePath] into PhoneGap compatible JavaScript in [outputDirectoryPath] */
Future<bool> handheldBuild(String inputFilePath, String outputDirectoryPath) {
  var inputFile = new File(inputFilePath);
  assert(inputFile.existsSync());
  
  var outDir = new Directory(outputDirectoryPath);
  assert(outDir.existsSync());
  return _build(inputFile, outDir);
}

Future<bool> _build(File inputFile, Directory outputDir) {
  var completer = new Completer<bool>();
  
  var currentDir = Directory.current.path;
  var outPath = outDir.append('${inFile.filename}.js');
  var args = ['--out=$outPath', inFile.toNativePath()];
  
  new Directory.fromPath(outDir).createSync(recursive: true);

  _logger.debug('running $_dart2jsPath args: $args current dir: $currentDir');
  Process.run(_dart2jsPath.toNativePath(), args, workingDirectory:currentDir).catchError ((e) {
    _logger.error('error: $e');
    throw e;
  }).then((ProcessResult result) {
    _logger.debug('dart2js exitCode: ${result.exitCode}');
    completer.complete(result.exitCode == 0);
  });
  
  return completer.future;
}

Path get _dart2jsPath => new Path(new Options().executable).directoryPath.append('dart2js');

Logger get _logger => LoggerFactory.getLogger('handheld_builder');

