// Copyright (c) 2013, the project authors. Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed 
// by a Apache license that can be found in the LICENSE file.

part of handheld;

/** Interact with the devices WebSQL database */
abstract class DeviceDatabase {
  /** Run a single database query */
  Future<SQLResult> executeSql(String sql);

  /** Run multiple database queries */
  SQLBatch batchSql(String sql);

  /**
   * Atomically verify and update the schema version number
   *
   * The future retuned contains the new version number of the database after the update
   */
  Future<String> changeVersion(String expectedCurrentVersion, String newVersion);

  /** Close the database connection */
  close();
}

/** The result of running a SQL query */
abstract class SQLResult {
  /** The query executed */
  String get query;

  /** The result set of the query */
  List<Map> get result;

  /** Convinece function to get entry in SQL result */
  Map operator[](int index);

  /** Convinece function to loop over each entry in the SQL result */
  forEach(f(Map data));
}

/** Batch and execute SQL commands together */
abstract class SQLBatch {
  /** Batch SQL commands to run against database. */
  SQLBatch batchSql(String sql);

  /** Execute all SQL commands in batch and return their result. */
  Future<SQLBatchResult> executeBatch();
}

/** The result of running a batch of SQL queries. */
abstract class SQLBatchResult {
  List<SQLResult> get results;

  /** Convinece function to get entry in batch result. */
  SQLResult operator[](int index);

  /** Convinece function to loop over each entry in the batch result. */
  forEach(f(SQLResult result));
}
