// Copyright (c) 2013, the project authors. Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed 
// by a Apache license that can be found in the LICENSE file.

part of handheld_js_impl;

class DeviceAccelerometerJs implements DeviceAccelerometer {
  @override
  Future<DeviceAcceleration> get currentAcceleration {
    var completer = new Completer<DeviceAcceleration>();
    
    onSuccess(var acceleration) {
      var deviceAcceleration = _mapDeviceAcceleration(acceleration);
      completer.complete(deviceAcceleration);
    }
    onError() => completer.completeError("getting device acceleration failed");
    
    js.context.navigator.accelerometer.getCurrentAcceleration(onSuccess, onError);
    return completer.future;
  }
  
  @override
  DeviceAccelerationStream watchAcceleration(int frequency) => new JsDeviceAccelerationStream(frequency);
}

class JsDeviceAccelerationStream implements DeviceAccelerationStream  {
  JsDeviceAccelerationStream(int frequency) {
    onSuccess(var acceleration) {
      var deviceAcceleration = _mapDeviceAcceleration(acceleration);
      if(_listener != null) {
        _listener(deviceAcceleration);
      }
    }
    onError() => throw "watching device acceleration failed";
    
    var options = js.map({ 'frequency': frequency });
    _watchId = js.context.navigator.accelerometer.watchAcceleration(onSuccess, onError, options);
  }
  
  @override
  listen(DeviceAccelerationListener listener) => _listener = listener;
  
  @override
  close() {
    if(_watchId != null) {
      js.context.navigator.accelerometer.clearWatch(_watchId);
    }
  }
  
  String _watchId;
  DeviceAccelerationListener _listener;
}

DeviceAcceleration _mapDeviceAcceleration(var acceleration) {
  var x = acceleration.x;
  var y = acceleration.y;
  var z = acceleration.z;
  var timestamp = acceleration.timestamp;
  // print("timestamp $timestamp");
  // TODO convert timestamp
  return new DeviceAcceleration(x,y,z, new DateTime.now());
}


